@@ -16,17 +16,12 @@
 #
 ##############################################################################
 from ..connector import connector as conn
-import req_body, ucfe_requirement
+from . import req_body, ucfe_requirement
 import math
 from datetime import datetime
 from random import choice
 from xml.etree.ElementTree import Element, SubElement, tostring
 
-
-import sys
-reload(sys)
-sys.setdefaultencoding('UTF8')
-
 # Cuidado de respetar el orden !!!!!!!!!
 # En caso de cambiar el orden podria generar
 # problemas al validar comprobantes en Uruware
@@ -47,6 +42,7 @@ INVOICE_INDICATOR = {
     'free_delivery': 5,
     'no_invoiceable': 6,
     'negative_no_invoiceable': 7,
+    'refund_picking': 8,
     'refund': 9,
     'exportation': 10,
     'tax': 11,
@@ -165,9 +161,11 @@ class Manager(object):
         self.build_Receptor(Encabezado, type)
         self.build_Totales(Encabezado, type)
         self.build_Detalles(root, type)
+        if hasattr(self.comprobante, 'invoice_discount_lines'):
+            self.build_DescuentosRecargos(root, type)
         self.build_Referencias(root)
         self.build_Compl_Fiscal_Data(root)
-        xml_msg = '<CFE version="1.0" xmlns="http://cfe.dgi.gub.uy">' + tostring(root) + '</CFE>'
+        xml_msg = '<CFE version="1.0" xmlns="http://cfe.dgi.gub.uy">{}</CFE>'.format(tostring(root, 'unicode'))
         return xml_msg
 
     def build_IdDoc(self, Encabezado):
@@ -189,6 +187,12 @@ class Manager(object):
         if self.comprobante.sale_mode_code: SubElement(IdDoc, "ModVenta").text = str(self.comprobante.sale_mode_code if self.comprobante.sale_mode_code else 'N/A')
         if self.comprobante.transport_route_code: SubElement(IdDoc, "ViaTransp").text = str(self.comprobante.transport_route_code)
 
+        if hasattr(self.comprobante, 'owner'): SubElement(IdDoc, "IndPropiedad").text = str(self.comprobante.owner)
+        if hasattr(self.comprobante, 'owner_vat_code'): SubElement(IdDoc, "TipoDocProp").text = str(self.comprobante.owner_vat_code)
+        if hasattr(self.comprobante, 'owner_country_code'): SubElement(IdDoc, "CodPaisProp").text = str(self.comprobante.owner_country_code)
+        if hasattr(self.comprobante, 'owner_vat'): SubElement(IdDoc, "DocProp").text = str(self.comprobante.owner_vat)
+        if hasattr(self.comprobante, 'owner_partner_name'): SubElement(IdDoc, "RznSocProp").text = str(self.comprobante.owner_partner_name)
+
         return IdDoc
 
     def build_Emisor(self, Encabezado):
@@ -216,7 +220,7 @@ class Manager(object):
             'eTck': self.build_local_receptor,
             'eBoleta': self.build_local_receptor,
             'eResg': self.build_local_receptor,
-            'eRem': self.build_local_receptor,
+            'eRem': self.build_local_receptor_rem,
             'eRem_Exp': self.build_exportation_receptor,
         }
 
@@ -235,6 +239,8 @@ class Manager(object):
         SubElement(Receptor, "CiudadRecep").text = str(self.comprobante.partner.city or '')
         SubElement(Receptor, "DeptoRecep").text = str(self.comprobante.partner.state or '')
         SubElement(Receptor, "PaisRecep").text = str(self.comprobante.partner.country or '')
+        if hasattr(self.comprobante, 'identificator'):
+            SubElement(Receptor, "CompraID").text = str(self.comprobante.identificator or '')
         if self.comprobante.partner.zip: SubElement(Receptor, "CP").text = str(self.comprobante.partner.zip or '')
 
     def build_local_receptor(self, Receptor):
@@ -246,6 +252,22 @@ class Manager(object):
         SubElement(Receptor, "DirRecep").text = str(self.comprobante.partner.street or '')
         SubElement(Receptor, "CiudadRecep").text = str(self.comprobante.partner.city or '')
         SubElement(Receptor, "DeptoRecep").text = str(self.comprobante.partner.state or '')
+        if hasattr(self.comprobante, 'identificator'):
+            SubElement(Receptor, "CompraID").text = str(self.comprobante.identificator or '')
+
+    def build_local_receptor_rem(self, Receptor):
+
+        SubElement(Receptor, "TipoDocRecep").text = str(self.comprobante.partner.vat.code or '')
+        SubElement(Receptor, "CodPaisRecep").text = str(self.comprobante.partner.country_code or '')
+        SubElement(Receptor, self.get_doc_recep_type(self.comprobante.partner.country_code)).text = str(self.comprobante.partner.vat.number or '')
+        SubElement(Receptor, "RznSocRecep").text = str(self.comprobante.partner.name)
+        SubElement(Receptor, "DirRecep").text = str(self.comprobante.partner.street or '')
+        SubElement(Receptor, "CiudadRecep").text = str(self.comprobante.partner.city or '')
+        SubElement(Receptor, "DeptoRecep").text = str(self.comprobante.partner.state or '')
+        SubElement(Receptor, "PaisRecep").text = str(self.comprobante.partner.country or '')
+        if hasattr(self.comprobante, 'identificator'):
+            SubElement(Receptor, "CompraID").text = str(self.comprobante.identificator or '')
+        if self.comprobante.partner.zip: SubElement(Receptor, "CP").text = str(self.comprobante.partner.zip or '')
 
     def build_Totales(self, Encabezado, type):
         """
@@ -279,6 +301,7 @@ class Manager(object):
         SubElement(Totales, "TpoMoneda").text = str(self.comprobante.currency) or ''
         if self.comprobante.currency_rate: SubElement(Totales, "TpoCambio").text = self._format_tipo_cambio(self.comprobante.currency_rate)
         if self.get_mnt_non_taxable() or self.get_mnt_exempt(): SubElement(Totales, "MntNoGrv").text = self._format_amount(self.get_mnt_non_taxable() + self.get_mnt_exempt())
+        if self.get_mnt_exportation(): SubElement(Totales, "MntExpoyAsim").text = self._format_amount(self.get_mnt_exportation())
         if self.get_mnt_base_iva_tasa_min(): SubElement(Totales, "MntNetoIvaTasaMin").text = self._format_amount(self.get_mnt_base_iva_tasa_min())
         if self.get_mnt_base_iva_tasa_basic(): SubElement(Totales, "MntNetoIVATasaBasica").text = self._format_amount(self.get_mnt_base_iva_tasa_basic())
         if self.get_mnt_base_iva_tasa_min(): SubElement(Totales, "IVATasaMin").text = TAXES.get('minimum')
@@ -307,7 +330,7 @@ class Manager(object):
             SubElement(retentions[ret_nro], "ValRetPerc").text = self._format_amount(ret.get_amount() * factor)
 
     def totales_eRem(self, Totales):
-        SubElement(Totales, "CantLinDet").text = str(len(self.comprobante.invoice_lines))
+        SubElement(Totales, "CantLinDet").text = str(len(self.comprobante.picking_lines))
 
     def build_Compl_Fiscal_Data(self, root):
         if hasattr(self.comprobante, 'fiscal_data'):
@@ -371,14 +394,44 @@ class Manager(object):
 
         DETALLES_BUILDER.get(type)(Detalle)
 
+    def build_DescuentosRecargos(self, root, type):
+
+        DescuentoRecargo = SubElement(root, "DscRcgGlobal")
+        DESCUENTOS_BUILDER = {
+            'eFact_Exp': self.descuento_eFact,
+            'eFact': self.descuento_eFact,
+            'eTck': self.descuento_eFact,
+        }
+
+        DESCUENTOS_BUILDER.get(type)(DescuentoRecargo)
+
+    def descuento_eFact(self, DescuentoRecargo):
+        item_nro = 0
+        Item = {}
+        for line in self.comprobante.invoice_discount_lines:
+            item_nro += 1
+            Item[item_nro] = SubElement(DescuentoRecargo, "DRG_Item")
+            SubElement(Item[item_nro], "NroLinDR").text = str(item_nro)
+            # Los descuentos se envia D en los recargos se envia R
+            SubElement(Item[item_nro], "TpoMovDR").text = 'D'
+            # Si el monto es un monto en pesos se envia 1 un porcentaje se envia 2
+            SubElement(Item[item_nro], "TpoDR").text = '1'
+            # Descripcion
+            SubElement(Item[item_nro], "GlosaDR").text = str(line.description)
+            # Valor de descuento
+            SubElement(Item[item_nro], "ValorDR").text = str(line.amount)
+            # Indicador de facturación
+            SubElement(Item[item_nro], "IndFactDR").text = str(line.invoice_indicator)
+
     def detalle_eRem(self, Detalle):
         item_nro = 0
         Item = {}
-        for line in self.comprobante.invoice_lines:
+        for line in self.comprobante.picking_lines:
             item_nro += 1
             Item[item_nro] = SubElement(Detalle, "Item")
             SubElement(Item[item_nro], "NroLinDet").text = str(item_nro)
-            if line.invoice_indicator: SubElement(Item[item_nro], "IndFact").text = str(line.invoice_indicator)
+            if hasattr(self.comprobante, 'refund_info'):
+                SubElement(Item[item_nro], "IndFact").text = str(INVOICE_INDICATOR.get('refund_picking'))
             SubElement(Item[item_nro], "NomItem").text = str(line.description)
             if hasattr(line, 'description_extra'):
                 SubElement(Item[item_nro], "DscItem").text = str(line.description_extra)
@@ -436,28 +489,37 @@ class Manager(object):
         return '{:.2f}'.format(rate)
 
     def _get_uuid_cfe(self):
-        longitud = 12
-        valores = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ<=>@#%&+"
-        p = ""
-        p = p.join([choice(valores) for i in range(longitud)])
-        return p
+        uuid = self.comprobante.uuid if hasattr(self.comprobante, 'uuid') else False
+        if not uuid:
+            longitud = 12
+            valores = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ<=>@#%&+"
+            uuid = ""
+            uuid = uuid.join([choice(valores) for i in range(longitud)])
+        return uuid
 
     def get_mnt(self, indicator):
         total = sum([line.amount for line in self.comprobante.invoice_lines if
                      str(line.invoice_indicator) == str(indicator)])
-        return total
+        total_discount = sum([line_discount.amount for line_discount in self.comprobante.invoice_discount_lines if
+                     str(line_discount.invoice_indicator) == str(indicator)]) if hasattr(self.comprobante, 'invoice_discount_lines') else 0
+        return total - total_discount if total else 0
 
     def get_tax(self, indicator):
         total = sum([line.tax for line in self.comprobante.invoice_lines if
                      str(line.invoice_indicator) == str(indicator)])
-        return total
+        tax_discount = sum([line_discount.tax for line_discount in self.comprobante.invoice_discount_lines if
+                            str(line_discount.invoice_indicator) == str(indicator)]) if hasattr(self.comprobante, 'invoice_discount_lines') else 0
+        return total - tax_discount if total else 0
 
     def get_mnt_base(self, indicator):
         if self.comprobante.include_tax:
             total = sum([line.amount - line.tax for line in self.comprobante.invoice_lines if str(line.invoice_indicator) == str(indicator)])
         else:
             total = sum([line.amount for line in self.comprobante.invoice_lines if str(line.invoice_indicator) == str(indicator)])
-        return total
+        total_discount = sum([line_discount.amount for line_discount in self.comprobante.invoice_discount_lines if
+                              str(line_discount.invoice_indicator) == str(indicator)]) if\
+            hasattr(self.comprobante,'invoice_discount_lines') else 0
+        return total - total_discount if total else 0
 
     def get_mnt_non_invoiceable(self):
         return self.get_mnt(INVOICE_INDICATOR.get('no_invoiceable'))
@@ -495,7 +557,6 @@ class Manager(object):
         return self.get_total_amount() + self.get_mnt_non_invoiceable()
 
     def get_total_retention(self):
-
         return sum([ret.get_amount() for ret in self.comprobante.retentions])
 
 # vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
